import os
import sys
import subprocess

from snipster.globalVars import sourceDir, snippetListFile, version, help
from snipster.Snippet import Snippet
from snipster.SnippetList import showSnippetList, lookupSnippetPath
from snipster.Sourcer import sourceSnippets

def main():
    parseCLIArgs(sys.argv[1:])
    exit(0)

def parseCLIArgs(cliArgs):
    if len(cliArgs) == 0 or cliArgs[0] == "-h" or cliArgs[0] == "--help":
        print(help)
        return
    if cliArgs[0] == "-v" or cliArgs[0] == "--version":
        print(version)
        return

    if cliArgs[0] == "source":
        sourceSnippets()
        return
    elif cliArgs[0] == "list":
        showSnippetList(cliArgs[1:])
        return

    else:
        lastArg = cliArgs[len(cliArgs)-1]
        if (len(cliArgs[0]) == 3 and cliArgs[0][2] == "f") or (len(cliArgs) == 3 and cliArgs[1] == "-f"):
            snippetFilePath = sourceDir + lastArg
        else:
            try:
                int(lastArg)
            except ValueError:
                print(help)
                return

            snippetFilePath = lookupSnippetPath(lastArg)

    if cliArgs[0][:2] == "-o":
        try:
            Snippet(snippetFilePath).display()
        except Exception as e:
            print(str(e))

    elif cliArgs[0][:2] == "-c":
        try:
            Snippet(snippetFilePath).copyToClipboard()
        except Exception as e:
            print(str(e))

    elif cliArgs[0][:2] == "-e":
        openInEditor(snippetFilePath)


def openInEditor(snippetFilePath):
    editor = os.environ.get('VISUAL') or os.environ.get('EDITOR') or False

    # assert that the editor is set
    if editor == False:
        print("Please set VISUAL or EDITOR in your bashrc to be able to create/edit snippets.")
        exit(1)

    try:
        subprocess.run(editor.split(" ") + [snippetFilePath])
    except OSError:
        print('Could not launch ' + editor)
        exit(1)



import os

from snipster.globalVars import sourceDir, snippetListFile
from snipster.Snippet import Snippet

snippetList = []

def sourceSnippets():
    if not os.path.exists(sourceDir):
        print("Initializing snipster")
        os.makedirs(sourceDir)
    walkDirectories(sourceDir)
    saveSnippetList(sourceDir)
    print("Finished sourcing snippets.")


def walkDirectories(basePath):
    allTheFiles = []
    subDirectories = []
    for(_, dirnames, filename) in os.walk(basePath):
        allTheFiles.extend(filename)
        subDirectories.extend(dirnames)
        break

    addSnippetsToList(allTheFiles, basePath)
    for directory in subDirectories:
        walkDirectories(basePath + directory + "/")


def addSnippetsToList(allTheFiles, basePath):
    newSnippets = []
    for file in allTheFiles:
        if file[0:2] == "__":
            continue
        try:
            snippet = Snippet(basePath + file)
            if int(snippet.id) == 0:
                newSnippets.append(snippet)
            else:
                snippetList.append(snippet)
        except Exception as e:
            print("Snippet " + str(file) + " not added to list: " + str(e))

    for snippet in newSnippets:
        newIndex = len(snippetList)+1
        while existsSnippet(newIndex):
            newIndex += 1
        snippet.setId(newIndex)
        snippetList.append(snippet)


def existsSnippet(id):
    for snippet in snippetList:
        if int(snippet.id) == id:
            return True
    return False


# formatting: id;title;lang;tag1,tag2;filepath
def saveSnippetList(sourceDir):
    listFile = open(sourceDir + snippetListFile, "w")
    for snippet in snippetList:
        listFile.write(str(snippet.id) + ";")
        listFile.write(snippet.title + ";")
        listFile.write(snippet.language + ";")
        taglist = ""
        for tag in snippet.tags:
            taglist += tag + ", "
        listFile.write(taglist.rstrip(", ") + ";")
        listFile.write(snippet.path + "\n")




from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters.terminal256 import Terminal256Formatter
import pyperclip

class Snippet:
    path = ""
    id = 0
    tags = []
    language = ""
    title = ""
    description = ""
    codeLanguage = ""
    code = ""

    def __init__(self, path):
        try:
            with open(path) as file:
                self.parse(file)
        except FileNotFoundError:
            raise FileNotFoundError("File " + path + " not found.")

        self.path = path


    def parse(self, file):
        # validate file formatting
        if file.readline() != "---\n":
            raise Exception("File not a valid snippet")
        currentLine = file.readline()

        # parse frontmatter
        while currentLine != "---\n":
            key,value = currentLine.split(":",1)
            self.assignKeyValues(key, value.lstrip(" ").rstrip("\n"))
            currentLine = file.readline()
        currentLine = file.readline()

        # parse description
        while currentLine[:3] != "```":
            self.description += currentLine
            currentLine = file.readline()
        self.description = self.description.lstrip("\n").rstrip("\n")

        # parse actual language of the file (for syntax highlighting)
        self.codeLanguage = currentLine.lstrip("```").rstrip("\n")
        currentLine = file.readline()

        # parse actual file (the code)
        while currentLine != "```\n":
            self.code += currentLine
            currentLine = file.readline()
        self.code = self.code.rstrip("\n")


    def assignKeyValues(self, key, values):
        if key == "id":
            self.id = int(values)
        elif key == "tags":
            self.tags = [tag.lstrip(" ").rstrip(" ") for tag in values.split(",")]
        elif key == "title":
            self.title = values.lstrip("\"").rstrip("\"")
        elif key == "lang":
            self.language = values

    def setId(self, id):
        with open(self.path, "r+") as file:
            file.readline()
            oldFile = file.read()
            file.seek(0,0)
            file.write("---\nid: " + str(id) + "\n" + oldFile)
        self.id = id

    def display(self):
        lexer = get_lexer_by_name(self.codeLanguage, stripall=True)
        formatter = Terminal256Formatter()
        code = highlight(self.code, lexer, formatter)
        print("#" + str(self.id) + ": \033[1m" + self.title + "\033[0m\n")
        print(self.description + "\n")
        print(code)


    def copyToClipboard(self):
        pyperclip.copy(self.code)
        print("Copied snippet #" + str(self.id) + " to the clipboard.")





import os
import csv

from snipster.globalVars import sourceDir, snippetListFile
from snipster.Snippet import Snippet
from tabulate import tabulate

snippetList = []
tags = []
keywords = []
languages = []
filteredSnippetList = []
tagHits = []
keywordHits = []
languageHits = []

def lookupSnippetPath(id):
    openSnippetList()
    return findSnippet(id)

def showSnippetList(filters):
    global filteredSnippetList
    openSnippetList()
    if filters != []:
        setUpFilters(filters)
        filterSnippets()
    else:
        filteredSnippetList = snippetList
    printSnippets()

def filterSnippets():
    for snippet in snippetList:
        if len(tags) != 0:
            filterByTag(snippet)
        if len(keywords) !=0:
            filterByKeyword(snippet)
        if len(languages) != 0:
            filterByLanguage(snippet)
    unionFilterHits()

def unionFilterHits():
    tempList = []
    for snippet in tagHits:
        if (len(keywords) == 0 or snippet in keywordHits) and (len(languages) == 0 or snippet in languageHits):
            tempList.append(snippet)
    for snippet in keywordHits:
        if (len(tags) == 0 or snippet in tagHits) and (len(languages) == 0 or snippet in languageHits):
            tempList.append(snippet)
    for snippet in languageHits:
        if (len(tags) == 0 or snippet in tagHits) and (len(keywords) == 0 or snippet in keywordHits):
            tempList.append(snippet)

    seen = []
    for snippet in tempList:
        if snippet not in seen:
            filteredSnippetList.append(snippet)
            seen.append(snippet)

def filterByTag(snippet):
    for tag in tags:
        if tag.lower() in snippet[3].lower():
            tagHits.append(snippet)
            return

def filterByKeyword(snippet):
    for keyword in keywords:
        if keyword.lower() in snippet[1].lower():
            keywordHits.append(snippet)
            return

def filterByLanguage(snippet):
    for language in languages:
        if language.lower() in snippet[2].lower():
            languageHits.append(snippet)
            return


def setUpFilters(filters):
    for value in filters:
        if value == "-t":
            filterType = "tags"
            continue
        elif value == "-k":
            filterType = "keywords"
            continue
        elif value == "-l":
            filterType = "language"
            continue

        if filterType == "tags":
            tags.append(value)
        if filterType == "keywords":
            keywords.append(value)
        if filterType == "language":
            languages.append(value)


def printSnippets():
    headers = ["ID", "Title", "Language", "Tags", "Filename"]
    table = []
    if len(filteredSnippetList) == 0:
        print("No matches.")
        return
    if len(filteredSnippetList) == 1:
        Snippet(filteredSnippetList[0][4]).display()
        return
    for snippet in filteredSnippetList:
        filePathIndex = len(snippet)-1
        snippet[filePathIndex] = snippet[filePathIndex][len(sourceDir):]
        table.append(snippet)
    print(tabulate(filteredSnippetList, headers=headers, tablefmt="pipe"))


def openSnippetList():
    try:
        with open(sourceDir + snippetListFile) as snippetCSV:
            snippetListCSVFile = csv.reader(snippetCSV, delimiter= ";")
            for row in snippetListCSVFile:
                snippetList.append(row)
    except FileNotFoundError:
        print("Didn't find a snippet list file. You can create one by using the source command ('snipster source')")
        exit(1)

def findSnippet(id):
    for snippet in snippetList:
        if int(snippet[0]) == int(id):
            return snippet[len(snippet)-1]
    print("No snippet with id " + str(id) + " found.")
    exit(1)




from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name         = 'snipster-py',
    version      = '1.0.3',
    author       = 'Sophie Au',
    author_email = 'some.person@web.de',
    license      = 'MIT',
    description  = 'A simple command line snippet manager',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    keywords     = ['snippet', 'snippets-manager', 'command-line-tool'],
    url          = 'https://github.com/sophieau/snipster',

    packages     = ['snipster'],
    zip_safe     = False,
    entry_points = {
        'console_scripts': ['snipster=snipster.__main__:main'],
    },
    python_requires  = '>=3.6',
    install_requires = [
        'Pygments >= 2.2.0',
        'pyperclip >= 1.6.0',
        'tabulate >= 0.8.2',
    ]
)



import os

sourceDir = str(os.path.expanduser("~/.snipster/"))

snippetListFile = "__snipster__.csv"

version = "1.0.3"

help = """Snipster.

Usage:
    snipster source
    snipster list
    snipster list [-t <tag>... | -l <language>... | -k <keyword>... ]...
    snipster (-c|-e|-o) [-f] <snippet-id>
    snipster -h | --help
    snipster -v | --version

Options:
    -h, --help      Show this screen.
    -v, --version   Show version.
    -t TAG...       Filter snippet list by tag.
    -l LANGUAGE...  Filter snippet list by language.
    -k KEYWORD...   Filter snippet list by keyword.
    -c SNIPPET-ID   Copy snippet to clipboard.
    -e SNIPPET-ID   Edit snippet.
    -o SNIPPET-ID   Open snippet
    -f              Select snippet from filename intead of ID.
"""
